importPackage(java.awt);
importPackage(java.awt.geom);
include(Resources.id('mtrsteamloco:scripts/display_helper.js'));

let slotCfg = {
    version: 1,
    texSize: [1024, 128],
    slots: [
        {
            name: 'head',
            texArea: [0, 0, 1024, 128],
            pos: [
                [
                    [-0.5, 0.92, 10.16],
                    [-0.5, 0.8, 10.16],
                    [0.52, 0.8, 10.16],
                    [0.52, 0.92, 10.16],
                ],
            ],
            offsets: [[0, 0, 0]],
        },
    ],
};

let slotReversedCfg = {
    version: 1,
    texSize: [1024, 128],
    slots: [
        {
            name: 'head-reversed',
            texArea: [0, 0, 1024, 128],
            pos: [
                [
                    [0.5, 0.92, -10.16],
                    [0.5, 0.8, -10.16],
                    [-0.52, 0.8, -10.16],
                    [-0.52, 0.92, -10.16],
                ],
            ],
            offsets: [[0, 0, 0]],
        },
    ],
};

let dhBase = new DisplayHelper(slotCfg);
let dhReversedBase = new DisplayHelper(slotReversedCfg);

create_queue.push(function (ctx, state, train) {
    state.pisRateLimit = new RateLimit(0.05);
    state.dh = dhBase.create();
    state.dhReversed = dhReversedBase.create();
    state.routeID = Math.floor(Math.random() * 100);
});

dispose_queue.push(function (ctx, state, train) {
    state.dh.close();
    state.dhReversed.close();
});

var EMUFont = Resources.readFont(
    Resources.idRelative('mtr:minsk/fonts/arial.ttf')
);

render_queue.push(function render(ctx, state, train) {
    if (state.pisRateLimit.shouldUpdate()) {
        let g;

        let EMUText = getDestinationStation(train);

        [
            { dh: state.dh, graphicsName: 'head' },
            { dh: state.dhReversed, graphicsName: 'head-reversed' },
        ].forEach((element) => {
            let dh = element.dh;
            g = dh.graphicsFor(element.graphicsName);
            g.setColor(Color.BLACK);
            g.fillRect(0, 0, 1024, 128);
            g.setColor(Color.ORANGE);
            g.setFont(EMUFont.deriveFont(Font.PLAIN, 110));
            let fontMetrics = g.getFontMetrics();
            let routeWidth = 0;
            let textWidth = fontMetrics.stringWidth(EMUText);
            g.drawString(
                EMUText,
                routeWidth + (1024 - routeWidth - textWidth) / 2,
                100
            );
            dh.upload();
        });
    }

    ctx.drawCarModel(state.dh.model, 0, null);
    ctx.drawCarModel(state.dhReversed.model, train.trainCars() - 1, null);
});

function getDestinationStation(train) {
    let stationList = train.getThisRoutePlatforms();
    let lastPlatform = stationList[stationList.size() - 1];
    if (lastPlatform === undefined) return 'нет посадки';
    return lastPlatform.destinationName.split('\\|')[0].toUpperCase();
}
